import json
import requests

import openai_proxy
from openai_proxy.utils import token_estimator


def authenticate():
    # TODO: Authorize public users
    if openai_proxy.username == "" or openai_proxy.username is None \
            or openai_proxy.course_id == "" or openai_proxy.course_id is None \
            or openai_proxy.access_key == "" or openai_proxy.access_key is None \
            or openai_proxy.access_token == "" or openai_proxy.access_token is None:
        return "Please set your username, courseId, accessKey, and accessToken"
    return False


class Completion:
    @staticmethod
    def create(prompt,
               engine="babbage",
               temperature=0.7,
               max_tokens=200,
               top_p=1,
               frequency_penalty=0,
               presence_penalty=0,
               stop='',
               n=1
               ):
        error = authenticate()
        if error:
            return error

        body = {
            "username": openai_proxy.username,
            "courseId": openai_proxy.course_id,
            "accessKey": openai_proxy.access_key,
            "accessToken": openai_proxy.access_token,
            "prompt": prompt,
            "max_tokens": max_tokens,
            "engine": engine,
            "temperature": temperature,
            "top_p": top_p,
            "frequency_penalty": frequency_penalty,
            "presence_penalty": presence_penalty,
            "stop": stop,
            "n": n
        }
        r = requests.post('http://openai-proxy.herokuapp.com/b/request/openai/completion', json=body)
        response = json.loads(r.text)
        if response['status'] == 'success':
            return response['response']
        else:
            return response

    @staticmethod
    def price(prompt,
              engine="babbage",
              max_tokens=200,
              n=1
              ):

        body = {
            "prompt": prompt,
            "max_tokens": max_tokens,
            "engine": engine,
            "n": n
        }

        return {
            "status": "success",
            "price": token_estimator.price_calculator_completion(body)
        }



import json
import requests

import openai_proxy
from openai_proxy.utils import token_estimator


def authenticate():
    # TODO: Authorize public users
    if openai_proxy.username == "" or openai_proxy.username is None \
            or openai_proxy.course_id == "" or openai_proxy.course_id is None \
            or openai_proxy.access_key == "" or openai_proxy.access_key is None \
            or openai_proxy.access_token == "" or openai_proxy.access_token is None:
        return "Please set your username, courseId, accessKey, and accessToken"
    return False


class ChatCompletion:
    @staticmethod
    def create(messages=[]):
        error = authenticate()
        if error:
            return error

        body = {
            "username": openai_proxy.username,
            "courseId": openai_proxy.course_id,
            "accessKey": openai_proxy.access_key,
            "accessToken": openai_proxy.access_token,
            "messages": messages
        }
        r = requests.post('http://openai-proxy.herokuapp.com/b/request/openai/chat', json=body)
        response = json.loads(r.text)
        if response['status'] == 'success':
            return response['response']
        else:
            return response

    @staticmethod
    def price(messages=[]):

        return {
            "status": "success",
            "price": token_estimator.price_calculator_chat(messages)
        }



from openai_proxy.utils.token_estimator import price_calculator_completion, \
    price_calculator_chat, price_calculator_embedding

__all__ = [
    "price_calculator_completion",
    "price_calculator_chat",
    "price_calculator_embedding",
]


price_per_token = {
    'text-ada-001': 0.0004 / 1000,
    'text-babbage-001': 0.0005 / 1000,
    'text-curie-001': 0.002 / 1000,
    'text-davinci-003': 0.02 / 1000,
    'ada-finetuned': 0.0016 / 1000,
    'babbage-finetuned': 0.0024 / 1000,
    'curie-finetuned': 0.0120 / 1000,
    'davinci-finetuned': 0.1200 / 1000,
    'ada-finetuned-training': 0.0004 / 1000,
    'babbage-finetuned-training': 0.0006 / 1000,
    'curie-finetuned-training': 0.0030 / 1000,
    'davinci-finetuned-training': 0.0300 / 1000,
    'text-embedding-ada-002': 0.0004 / 1000,
    'gpt-3.5-turbo': 0.002 / 1000,
}


def get_price_per_token(engine):
    if engine in price_per_token:
        return price_per_token[engine]
    else:
        if 'ada' in engine:
            return price_per_token['ada-finetuned']
        elif 'babbage' in engine:
            return price_per_token['babbage-finetuned']
        elif 'curie' in engine:
            return price_per_token['curie-finetuned']
        elif 'davinci' in engine:
            return price_per_token['davinci-finetuned']
        else:
            return 0.1200 / 1000


engine_max_tokens = {
    'text-ada-001': 2048,
    'text-babbage-001': 2048,
    'text-curie-001': 2048,
    'text-davinci-003': 4000,
    'ada-finetuned': 2048,
    'babbage-finetuned': 2048,
    'curie-finetuned': 2048,
    'davinci-finetuned': 4000,
    'ada-finetuned-training': 2048,
    'babbage-finetuned-training': 2048,
    'curie-finetuned-training': 2048,
    'davinci-finetuned-training': 4000,
    'text-embedding-ada-002': 2048,
    'gpt-3.5-turbo': 4096,
}


def get_engine_max_tokens(engine):
    if engine in engine_max_tokens:
        return engine_max_tokens[engine]
    else:
        if 'ada' in engine:
            return engine_max_tokens['ada-finetuned']
        elif 'babbage' in engine:
            return engine_max_tokens['babbage-finetuned']
        elif 'curie' in engine:
            return engine_max_tokens['curie-finetuned']
        elif 'davinci' in engine:
            return engine_max_tokens['davinci-finetuned']
        else:
            return 2048


__all__ = [
    "price_per_token",
    "get_price_per_token",
    "engine_max_tokens",
    "get_engine_max_tokens",
]



import json

from openai_proxy.utils import token_estimator


def get_training_data(filename):
    with open(filename, 'r') as json_file:
        json_list = list(json_file)

    data = ""
    for json_str in json_list:
        result = json.loads(json_str)
        data += result['prompt'] + "\n"
        data += result['completion'] + "\n"

    return data


class Price:
    @staticmethod
    def training(filename=None,
                 prompt=None,
                 engine="babbage",
                 ):
        if filename:
            if not filename.endswith(".jsonl"):
                return {
                    'status': 'error',
                    'error': 'Please provide a .jsonl file'
                }
            else:
                data = get_training_data(filename)
        elif prompt:
            data = prompt
        else:
            return {
                'status': 'error',
                'error': 'Please provide a .jsonl file or prompt'
            }

        if not engine.endswith("-finetuned-training"):
            engine = f"{engine}-finetuned-training"

        body = {
            "prompt": data,
            "engine": engine,
        }

        return {
            'status': 'success',
            "price": token_estimator.price_calculator(body)
        }



import os
import json
import requests

from openai_proxy.completion import Completion
from openai_proxy.price import Price
from openai_proxy.embeddings import Embedding
from openai_proxy.chat_completion import ChatCompletion


api_key = os.environ.get("OPENAI_API_KEY")
access_key = os.environ.get("OPENAI_PROXY_ACCESS_KEY")
access_token = os.environ.get("OPENAI_PROXY_ACCESS_TOKEN")

username = os.environ.get("OPENAI_PROXY_USERNAME")
course_id = os.environ.get("OPENAI_PROXY_COURSE_ID")

r = requests.get('http://openai-proxy.herokuapp.com/b/request/engines')
response = json.loads(r.text)

engines = [engine for engine in response if engine is not None]

__all__ = [
    "api_key",
    "access_key",
    "access_token",
    "username",
    "course_id",
    "engines",
    "Completion",
    "Price",
    "Embedding",
    "ChatCompletion"
]


from transformers import GPT2TokenizerFast
from openai_proxy.utils.constants import price_per_token, engine_max_tokens, get_price_per_token, get_engine_max_tokens


def token_counter(prompt):
    # counts tokens given input text
    #
    # input: text
    # output: length of tokens
    tokenizer = GPT2TokenizerFast.from_pretrained("gpt2")
    token_len = len(tokenizer(prompt)['input_ids'])

    return token_len


def parse_args(args):
    prompt = args['prompt'] if 'prompt' in args else ''
    engine = args['engine'] if 'engine' in args else 'babbage'
    max_tokens = args['max_tokens'] if 'max_tokens' in args else 500
    n = args['n'] if 'n' in args else 1
    total_tokens = args['total_tokens'] if 'total_tokens' in args else None

    return {
        "prompt": prompt,
        "engine": engine,
        "max_tokens": max_tokens,
        "n": n,
        "total_tokens": total_tokens,
    }


def price_calculator_completion(args):
    # given prompt, engine and max_tokens, calculate estimated price in dollar
    #
    # input:
    #     prompt: text
    #     engine: choose between ada, babbage, curie and davinci
    #     max_tokens: parameter that sets maximum length of model response, int
    # output: estimated price in dollar
    #
    # acknowledgement: this calculation is for completion model only.
    #                 best_of option is not considered.

    p_args = parse_args(args)
    prompt = p_args['prompt']
    engine = p_args['engine']
    max_tokens = p_args['max_tokens']
    n = p_args['n']
    total_tokens = p_args['total_tokens']

    token_len = token_counter(prompt)
    this_engine_max_tokens = get_engine_max_tokens(engine)
    this_price_per_token = get_price_per_token(engine)

    price_per_completion = min((token_len + int(max_tokens)), this_engine_max_tokens) * this_price_per_token

    if engine in ['ada', 'babbage', 'curie', 'davinci', 'ada-finetuned', 'babbage-finetuned', 'curie-finetuned',
                  'davinci-finetuned']:
        price = price_per_completion * n
    else:
        price = price_per_completion * 1

    # the prompt in this case is the entire training data (prompt + completion)
    if engine in ['ada-finetuned-training', 'babbage-finetuned-training', 'curie-finetuned-training',
                  'davinci-finetuned-training']:
        price = token_len * this_price_per_token

    if total_tokens:
        price = total_tokens * this_price_per_token

    return round(price, 10)


def price_calculator_chat(messages):
    cost = 0
    if 'total_tokens' in messages:
        cost = messages['total_tokens'] * get_price_per_token('gpt-3.5-turbo')
        return round(cost, 10)

    for message in messages:
        token_len = token_counter(message["content"])
        cost += token_len * get_price_per_token('gpt-3.5-turbo')
    return round(cost, 10)


def price_calculator_embedding(phrases):
    cost = 0
    for phrase in phrases:
        token_len = token_counter(phrase)
        cost += token_len * get_price_per_token('text-embedding-ada-002')
    return round(cost, 10)



from setuptools import setup, find_packages

VERSION = '0.2.0'
DESCRIPTION = 'A price proxy for the OpenAI API'
LONG_DESCRIPTION = 'This proxy enables better budgeting and cost management for making OpenAI API calls ' \
                   'including more transparency into pricing. Github repo here: ' \
                   'https://github.com/tomiwa123/openai-proxy-python'

# Setting up
setup(
    # the name must match the folder name 'openai_proxy'
    name="openai_proxy",
    version=VERSION,
    author="Ayotomiwa Akinyele",
    author_email="<ayotomiwa.akinyele@gmail.com>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['openai', 'transformers'],  # add any additional packages that
    # needs to be installed along with your package. Eg: 'caer'

    keywords=['python', 'openai', 'proxy', 'api', 'wrapper'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)